(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('ramda'), require('prop-types'), require('react-redux'), require('@redux-tools/utils')) :
typeof define === 'function' && define.amd ? define(['exports', 'react', 'ramda', 'prop-types', 'react-redux', '@redux-tools/utils'], factory) :
(global = global || self, factory(global.ReduxToolsInjectorsReact = {}, global.React, global.R, global.PropTypes, global.ReactRedux, global.ReduxToolsUtils));
}(this, function (exports, React, ramda, PropTypes, reactRedux, utils) { 'use strict';

var React__default = 'default' in React ? React['default'] : React;
PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var InjectorContext = React.createContext({
  store: null,
  namespace: null,
  withNamespace: ramda.identity
});

var mergeContextValues = ramda.mergeWith(ramda.flip(ramda.or));

var Provider =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(Provider, _Component);

  function Provider() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, [this].concat(args)) || this;
    var _this$props = _this.props,
        namespace = _this$props.namespace,
        store = _this$props.store,
        withNamespace = _this$props.withNamespace; // TODO: Handle changes in props and context values.

    _this.state = mergeContextValues(_this.context, {
      namespace: namespace,
      store: store,
      withNamespace: withNamespace
    });
    return _this;
  }

  var _proto = Provider.prototype;

  _proto.render = function render() {
    var children = this.props.children;
    var store = this.state.store;
    var providerElement = React__default.createElement(InjectorContext.Provider, {
      value: this.state
    }, children);
    return store ? React__default.createElement(reactRedux.Provider, {
      store: store
    }, providerElement) : providerElement;
  };

  return Provider;
}(React.Component);

Provider.propTypes = {
  children: PropTypes.node.isRequired,
  namespace: PropTypes.string,
  store: PropTypes.object,
  withNamespace: PropTypes.func
};
Provider.contextType = InjectorContext;

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function withInjectorContext(NextComponent) {
  var _class, _temp;

  return _temp = _class =
  /*#__PURE__*/
  function (_Component) {
    _inheritsLoose(WithInjectorContext, _Component);

    function WithInjectorContext() {
      var _this;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      _this = _Component.call.apply(_Component, [this].concat(args)) || this;
      _this.WrappedComponent = null;
      return _this;
    }

    var _proto = WithInjectorContext.prototype;

    _proto.render = function render() {
      var _this2 = this;

      return React__default.createElement(InjectorContext.Consumer, null, function (_ref) {
        var namespace = _ref.namespace,
            store = _ref.store,
            _ref$withNamespace = _ref.withNamespace,
            withNamespace = _ref$withNamespace === void 0 ? ramda.identity : _ref$withNamespace;

        // NOTE: React's reconciliation process would otherwise think that we're rendering
        // two different components (because we would be creating a new one each render).
        if (!_this2.WrappedComponent) {
          // TODO: Handle `withNamespace` changes.
          _this2.WrappedComponent = withNamespace(NextComponent);
        }

        return React__default.createElement(_this2.WrappedComponent, _extends({
          store: store
        }, namespace ? {
          namespace: namespace
        } : {}, _this2.props));
      });
    };

    return WithInjectorContext;
  }(React.Component), _class.displayName = "WithInjectorContext(" + utils.getDisplayName(NextComponent) + ")", _temp;
}

var counter = 0;

makeInjector.resetCounter = function () {
  return counter = 0;
};

var omitStore = ramda.omit(['store']);
function makeInjector(inject, eject) {
  return function (injectables, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        persist = _ref.persist,
        global = _ref.global;

    return function (NextComponent) {
      var Injector =
      /*#__PURE__*/
      function (_Component) {
        _inheritsLoose(Injector, _Component);

        function Injector(props) {
          var _this;

          _this = _Component.call(this, props) || this;
          _this.namespace = global ? null : props.namespace;
          _this.version = counter++;
          inject(props.store)(injectables, _this.namespace, _this.version);
          return _this;
        }

        var _proto = Injector.prototype;

        _proto.componentWillUnmount = function componentWillUnmount() {
          var store = this.props.store;

          if (!persist) {
            eject(store)(injectables, this.namespace, this.version);
          }
        };

        _proto.render = function render() {
          return React__default.createElement(NextComponent, omitStore(this.props));
        };

        return Injector;
      }(React.Component);

      Injector.propTypes = {
        namespace: PropTypes.string,
        store: PropTypes.object.isRequired
      };
      Injector.displayName = "Injector(" + utils.getDisplayName(NextComponent) + ")";
      return withInjectorContext(Injector);
    };
  };
}

exports.Provider = Provider;
exports.makeInjector = makeInjector;
exports.withInjectorContext = withInjectorContext;

Object.defineProperty(exports, '__esModule', { value: true });

}));
